@@ -1,9 +1,9 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-14/Applications/Cxx/gdcmgendir.cxx $
+  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-15/Applications/Cxx/gdcmgendir.cxx $
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -31,7 +31,7 @@
 void PrintVersion()
 {
   std::cout << "gdcmgendir: gdcm " << gdcm::Version::GetVersion() << " ";
-  const char date[] = "$Date: 2009-06-12 15:58:39 +0200 (Fri, 12 Jun 2009) $";
+  const char date[] = "$Date: 2010-06-23 18:15:58 +0200 (Wed, 23 Jun 2010) $";
   std::cout << date << std::endl;
 }
 
@@ -318,22 +318,24 @@ int main(int argc, char *argv[])
 
   int res = 0;
   unsigned int nfiles = 1;
+  gdcm::DICOMDIRGenerator gen;
   if( gdcm::System::FileIsDirectory(filename.c_str()) )
     {
     gdcm::Directory dir;
     nfiles = dir.Load(filename, recursive);
     filenames = dir.GetFilenames();
+    gen.SetRootDirectory( filename );
     }
   else
     {
     // should be all set !
     }
 
-  gdcm::DICOMDIRGenerator gen;
   gen.SetFilenames( filenames );
   gen.SetDescriptor( descriptor_str.c_str() );
   if( !gen.Generate() )
     {
+    std::cerr << "Problem during generation" << std::endl;
     return 1;
     }
 
