@@ -1,9 +1,9 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-14/Applications/Cxx/gdcmimg.cxx $
+  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-15/Applications/Cxx/gdcmimg.cxx $
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -48,8 +48,8 @@
 #include "gdcmSequenceOfFragments.h"
 #include "gdcmSystem.h"
 #include "gdcmReader.h"
-#include "gdcmImageWriter.h"
-#include "gdcmImageReader.h"
+#include "gdcmPixmapWriter.h"
+#include "gdcmPixmapReader.h"
 #include "gdcmFileMetaInformation.h"
 #include "gdcmDataSet.h"
 #include "gdcmAttribute.h"
@@ -110,7 +110,7 @@ void FillRegionWithColor(char *cp, const unsigned int *dims, const unsigned int
 void PrintVersion()
 {
   std::cout << "gdcmimg: gdcm " << gdcm::Version::GetVersion() << " ";
-  const char date[] = "$Date: 2009-10-24 21:19:36 +0200 (Sat, 24 Oct 2009) $";
+  const char date[] = "$Date: 2010-07-07 18:10:49 +0200 (Wed, 07 Jul 2010) $";
   std::cout << date << std::endl;
 }
 
@@ -125,7 +125,7 @@ void PrintHelp()
   std::cout << "  -i --input     Input filename" << std::endl;
   std::cout << "  -o --output    Output filename" << std::endl;
   std::cout << "Options:" << std::endl;
-  std::cout << "     --endian %s       Endianess (LSB/MSB)." << std::endl;
+  std::cout << "     --endian %s       Endianness (LSB/MSB)." << std::endl;
   std::cout << "  -d --depth %d        Depth (8/16/32)." << std::endl;
   std::cout << "     --sign %s         Pixel sign (0/1)." << std::endl;
   std::cout << "  -s --size %d,%d      Size." << std::endl;
@@ -254,13 +254,13 @@ bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string const &
   return true;
 }
 
-bool PopulateSingeFile( gdcm::ImageWriter & writer, gdcm::SequenceOfFragments *sq , gdcm::ImageCodec & jpeg, const char *filename )
+bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFragments *sq , gdcm::ImageCodec & jpeg, const char *filename )
 {
-      /*
-       * FIXME: when JPEG contains JFIF marker, we should only read them
-       * during header parsing but discard them when copying the JPG byte stream into 
-       * the encapsulated Pixel Data Element...
-       */
+  /*
+   * FIXME: when JPEG contains JFIF marker, we should only read them
+   * during header parsing but discard them when copying the JPG byte stream into 
+   * the encapsulated Pixel Data Element...
+   */
   std::ifstream is(filename);
   gdcm::TransferSyntax ts;
   bool b = jpeg.GetHeaderInfo( is, ts );
@@ -269,7 +269,7 @@ bool PopulateSingeFile( gdcm::ImageWriter & writer, gdcm::SequenceOfFragments *s
     return false;
     }
 
-  gdcm::Image &image = writer.GetImage();
+  gdcm::Pixmap &image = writer.GetPixmap();
   image.SetDimensions( jpeg.GetDimensions() );
   image.SetPixelFormat( jpeg.GetPixelFormat() );
   image.SetPhotometricInterpretation( jpeg.GetPhotometricInterpretation() );
@@ -308,11 +308,11 @@ bool PopulateSingeFile( gdcm::ImageWriter & writer, gdcm::SequenceOfFragments *s
   return true;
 }
 
-bool Populate( gdcm::ImageWriter & writer, gdcm::ImageCodec & jpeg, gdcm::Directory::FilenamesType const & filenames )
+bool Populate( gdcm::PixmapWriter & writer, gdcm::ImageCodec & jpeg, gdcm::Directory::FilenamesType const & filenames )
 {
   std::vector<std::string>::const_iterator it = filenames.begin();
   bool b = true;
-  gdcm::Image &image = writer.GetImage();
+  gdcm::Pixmap &image = writer.GetPixmap();
   image.SetNumberOfDimensions( 2 );
   if( filenames.size() > 1 )
     {
@@ -394,8 +394,9 @@ int main (int argc, char *argv[])
   int version = 0;
 
   gdcm::UIDGenerator uid;
-  std::string series_uid = uid.Generate();
-  std::string study_uid = uid.Generate();
+  // Too early for UID Generation
+  std::string series_uid; // = uid.Generate();
+  std::string study_uid; // = uid.Generate();
   while (1) {
     //int this_option_optind = optind ? optind : 1;
     int option_index = 0;
@@ -638,18 +639,6 @@ int main (int argc, char *argv[])
     return 1;
     }
 
-  if( !gdcm::UIDGenerator::IsValid( study_uid.c_str() ) )
-    {
-    std::cerr << "Invalid UID for Study UID: " << study_uid << std::endl;
-    return 1;
-    }
-
-  if( !gdcm::UIDGenerator::IsValid( series_uid.c_str() ) )
-    {
-    std::cerr << "Invalid UID for Series UID: " << series_uid << std::endl;
-    return 1;
-    }
-
   // Ok so we are about to write a DICOM file, do not forget to stamp it GDCM !
   gdcm::FileMetaInformation::SetSourceApplicationEntityTitle( "gdcmimg" );
   if( !rootuid )
@@ -672,6 +661,26 @@ int main (int argc, char *argv[])
       }
     gdcm::UIDGenerator::SetRoot( root.c_str() );
     }
+  if( study_uid.empty() )
+    {
+    study_uid = uid.Generate();
+    }
+  if( !gdcm::UIDGenerator::IsValid( study_uid.c_str() ) )
+    {
+    std::cerr << "Invalid UID for Study UID: " << study_uid << std::endl;
+    return 1;
+    }
+
+  if( series_uid.empty() )
+    {
+    series_uid  = uid.Generate();
+    }
+  if( !gdcm::UIDGenerator::IsValid( series_uid.c_str() ) )
+    {
+    std::cerr << "Invalid UID for Series UID: " << series_uid << std::endl;
+    return 1;
+    }
+
   // Debug is a little too verbose
   gdcm::Trace::SetDebug( debug );
   gdcm::Trace::SetWarning( warning );
@@ -705,7 +714,7 @@ int main (int argc, char *argv[])
         return 1;
         }
       gdcm::RAWCodec raw;
-      gdcm::ImageWriter writer;
+      gdcm::PixmapWriter writer;
       // Because the RAW stream is not self sufficient, we need to pass in some extra
       // user info:
       unsigned int dims[3] = {};
@@ -754,7 +763,7 @@ int main (int argc, char *argv[])
         return 1;
         }
       gdcm::RLECodec rle;
-      gdcm::ImageWriter writer;
+      gdcm::PixmapWriter writer;
       // Because the RLE stream is not self sufficient, we need to pass in some extra
       // user info:
       unsigned int dims[3] = {};
@@ -784,7 +793,7 @@ int main (int argc, char *argv[])
       || gdcm::System::StrCaseCmp(inputextension,".ppm") == 0 )
       {
       gdcm::PNMCodec pnm;
-      gdcm::ImageWriter writer;
+      gdcm::PixmapWriter writer;
       if( !Populate( writer, pnm, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer.GetFile().GetDataSet() ) ) return 1;
 
@@ -800,7 +809,7 @@ int main (int argc, char *argv[])
     if(  gdcm::System::StrCaseCmp(inputextension,".jls") == 0 )
       {
       gdcm::JPEGLSCodec jpeg;
-      gdcm::ImageWriter writer;
+      gdcm::PixmapWriter writer;
       if( !Populate( writer, jpeg, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer.GetFile().GetDataSet() ) ) return 1;
 
@@ -823,7 +832,7 @@ int main (int argc, char *argv[])
        * need to chop off all extra header information...
        */
       gdcm::JPEG2000Codec jpeg;
-      gdcm::ImageWriter writer;
+      gdcm::PixmapWriter writer;
       if( !Populate( writer, jpeg, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer.GetFile().GetDataSet() ) ) return 1;
 
@@ -847,7 +856,7 @@ int main (int argc, char *argv[])
       gdcm::PixelFormat pf = gdcm::PixelFormat::UINT8;
       if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign ) ) return 1;
       jpeg.SetPixelFormat( pf );
-      gdcm::ImageWriter writer;
+      gdcm::PixmapWriter writer;
       if( !Populate( writer, jpeg, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer.GetFile().GetDataSet() ) ) return 1;
 
@@ -863,7 +872,7 @@ int main (int argc, char *argv[])
     }
 // else safely assume that if no inputextension matched then it is a DICOM file
 
-  gdcm::ImageReader reader;
+  gdcm::PixmapReader reader;
   reader.SetFileName( filename );
   if( !reader.Read() )
     {
@@ -871,7 +880,7 @@ int main (int argc, char *argv[])
     return 1;
     }
 
-  const gdcm::Image &imageori = reader.GetImage();
+  const gdcm::Pixmap &imageori = reader.GetPixmap();
   const gdcm::File &file = reader.GetFile();
 
   if ( outputextension )
@@ -898,7 +907,7 @@ int main (int argc, char *argv[])
 
 // else safely assume that if no outputextension matched then it is a DICOM file
 
-  gdcm::ImageWriter writer;
+  gdcm::PixmapWriter writer;
   writer.SetFile( file );
   writer.SetImage( imageori );
   writer.SetFileName( outfilename );
@@ -910,7 +919,7 @@ int main (int argc, char *argv[])
     const gdcm::PixelFormat &pixeltype = imageori.GetPixelFormat();
     assert( imageori.GetNumberOfDimensions() == 2 || imageori.GetNumberOfDimensions() == 3 );
     unsigned long len = imageori.GetBufferLength();
-    gdcm::SmartPointer<gdcm::Image> image = new gdcm::Image;
+    gdcm::SmartPointer<gdcm::Pixmap> image = new gdcm::Pixmap;
     image->SetNumberOfDimensions( 2 ); // good default
     const unsigned int *dims = imageori.GetDimensions();
     if ( region[0] > region[1] 
@@ -941,6 +950,10 @@ int main (int argc, char *argv[])
       }
     image->SetPhotometricInterpretation( imageori.GetPhotometricInterpretation() );
     image->SetPixelFormat( imageori.GetPixelFormat() );
+    image->SetPlanarConfiguration( imageori.GetPlanarConfiguration() );
+    image->SetLUT( imageori.GetLUT() );
+    image->SetLossyFlag( imageori.IsLossy() );
+    // FIXME what is overlay is in pixel data ?
     gdcm::DataElement pixeldata( gdcm::Tag(0x7fe0,0x0010) );
     gdcm::ByteValue *bv = new gdcm::ByteValue();
     bv->SetLength( len );
@@ -969,8 +982,6 @@ int main (int argc, char *argv[])
 
     pixeldata.SetValue( *bv );
     image->SetDataElement( pixeldata );
-    image->SetSpacing( imageori.GetSpacing() );
-    image->SetSpacing(2, imageori.GetSpacing()[2] );
     const gdcm::TransferSyntax &ts = imageori.GetTransferSyntax();
     // FIXME: for now we do not know how to recompress the image...
     if( ts.IsExplicit() )
