@@ -1,9 +1,9 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-14/Applications/Cxx/gdcmpdf.cxx $
+  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-15/Applications/Cxx/gdcmpdf.cxx $
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -20,11 +20,17 @@
 #include "gdcmAttribute.h"
 #include "gdcmSystem.h"
 
+#ifdef GDCM_USE_SYSTEM_PODOFO
+#include <podofo/podofo.h>
+#endif
+
+#ifdef GDCM_USE_SYSTEM_POPPLER
 #include <poppler/poppler-config.h>
 #include <poppler/PDFDoc.h>
 #include <poppler/UnicodeMap.h>
 #include <poppler/PDFDocEncoding.h>
 #include <poppler/GlobalParams.h>
+#endif
 
 #include <string>
 
@@ -143,7 +149,7 @@ std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
 void PrintVersion()
 {
   std::cout << "gdcmpdf: gdcm " << gdcm::Version::GetVersion() << " ";
-  const char date[] = "$Date: 2009-05-19 15:59:08 +0200 (Tue, 19 May 2009) $";
+  const char date[] = "$Date: 2010-06-23 18:15:58 +0200 (Wed, 23 Jun 2010) $";
   std::cout << date << std::endl;
 }
 
