@@ -3,7 +3,7 @@
   Program: GDCM (Grassroots DICOM). A DICOM library
   Module:  $URL$
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -190,7 +190,102 @@ TransferSyntax Reader::GuessTransferSyntax()
   return ts;
 }
 
+namespace details
+{
+  class DefaultCaller
+  {
+  private:
+    gdcm::DataSet & m_dataSet;
+  public:
+    DefaultCaller(gdcm::DataSet &ds): m_dataSet(ds){}
+    template<class T1, class T2>
+      void ReadCommon(std::istream & is) const
+        {
+        m_dataSet.template Read<T1,T2>(is);
+        }
+    template<class T1, class T2>
+      void ReadCommonWithLength(std::istream & is, VL & length) const
+        {
+        m_dataSet.template ReadWithLength<T1,T2>(is,length);
+        }
+    static void Check(bool b, std::istream &stream) 
+      {
+      if( b ) assert( stream.eof() );
+      }
+  };
+
+  class ReadUpToTagCaller 
+  {
+  private:
+    gdcm::DataSet & m_dataSet;
+    const gdcm::Tag & m_tag;
+    std::set<gdcm::Tag> const & m_skipTags;
+  public:
+    ReadUpToTagCaller(gdcm::DataSet &ds,const gdcm::Tag & tag, std::set<gdcm::Tag> const & skiptags)
+    :
+    m_dataSet(ds),m_tag(tag),m_skipTags(skiptags)
+    {
+    }
+
+    template<class T1, class T2>
+      void ReadCommon(std::istream & is) const
+        {
+        m_dataSet.template ReadUpToTag<T1,T2>(is,m_tag,m_skipTags);
+        }
+    template<class T1, class T2>
+      void ReadCommonWithLength(std::istream & is, VL & length) const
+        {
+        m_dataSet.template ReadUpToTagWithLength<T1,T2>(is,m_tag,length);
+        }
+    static void Check(bool , std::istream &)  {}
+  }; 
+
+  class ReadSelectedTagsCaller 
+  {
+  private:
+    DataSet & m_dataSet;
+    std::set<Tag> const & m_tags;
+  public:
+    ReadSelectedTagsCaller(DataSet &ds, std::set<Tag> const & tags)
+      :
+    m_dataSet(ds),m_tags(tags)
+    {
+    }
+
+    template<class T1, class T2>
+    void ReadCommon(std::istream & is) const
+    {
+      m_dataSet.template ReadSelectedTags<T1,T2>(is,m_tags);
+    }
+    template<class T1, class T2>
+    void ReadCommonWithLength(std::istream & is, VL & length) const
+    {
+      m_dataSet.template ReadSelectedTagsWithLength<T1,T2>(is,m_tags,length);
+    }
+    static void Check(bool , std::istream &)  {}
+  }; 
+}
+
 bool Reader::Read()
+{
+  details::DefaultCaller caller(F->GetDataSet());
+  return InternalReadCommon(caller);
+}
+
+bool Reader::ReadUpToTag(const Tag & tag, std::set<Tag> const & skiptags)
+{
+  details::ReadUpToTagCaller caller(F->GetDataSet(),tag,skiptags);
+  return InternalReadCommon(caller);
+}
+
+bool Reader::ReadSelectedTags( std::set<Tag> const & selectedTags )
+{
+  details::ReadSelectedTagsCaller caller(F->GetDataSet(), selectedTags);
+  return InternalReadCommon(caller);
+}
+
+template <typename T_Caller>
+bool Reader::InternalReadCommon(const T_Caller &caller)
 {
   if( !Stream )
     {
@@ -232,16 +327,18 @@ std::istream &is = *Stream;
         }
       catch( std::exception &ex )
         {
+        (void)ex;
         // Weird implicit meta header:
         is.seekg(128+4, std::ios::beg );
         try
           {
           F->GetHeader().ReadCompat(is);
           }
-        catch( std::exception &ex )
+        catch( std::exception &ex2 )
           {
           // Ok I get it now... there is absolutely no meta header, giving up
           //hasmetaheader = false;
+          (void)ex2;
           }
         }
       }
@@ -287,7 +384,8 @@ std::istream &is = *Stream;
     zlib_stream::zip_istream gzis( is );
     // FIXME: we also know in this case that we are dealing with Explicit:
     assert( ts.GetNegociatedType() == TransferSyntax::Explicit );
-    F->GetDataSet().Read<ExplicitDataElement,SwapperNoOp>(gzis);
+    //F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperNoOp>(gzis,tag, skiptags);
+    caller.template ReadCommon<ExplicitDataElement,SwapperNoOp>(gzis);
     // I need the following hack to read: srwithgraphdeflated.dcm
     //is.clear();
     // well not anymore, see special handling of trailing \0 in:
@@ -304,12 +402,13 @@ std::istream &is = *Stream;
         {
         // There is no such thing as Implicit Big Endian... oh well
         // LIBIDO-16-ACR_NEMA-Volume.dcm
-        //F->GetDataSet().Read<ImplicitDataElement,SwapperDoOp>(is);
+        //F->GetDataSet().ReadUpToTag<ImplicitDataElement,SwapperDoOp>(is,tag, skiptags);
         throw "Virtual Big Endian Implicit is not defined by DICOM";
         }
       else
         {
-        F->GetDataSet().Read<ExplicitDataElement,SwapperDoOp>(is);
+        //F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperDoOp>(is,tag, skiptags);
+        caller.template ReadCommon<ExplicitDataElement,SwapperDoOp>(is);
         }
       }
     else // LittleEndian
@@ -318,7 +417,8 @@ std::istream &is = *Stream;
         {
         if( hasmetaheader && haspreamble )
           {
-          F->GetDataSet().Read<ImplicitDataElement,SwapperNoOp>(is);
+          //F->GetDataSet().ReadUpToTag<ImplicitDataElement,SwapperNoOp>(is,tag, skiptags);
+          caller.template ReadCommon<ImplicitDataElement,SwapperNoOp>(is);
           }
         else
           {
@@ -327,13 +427,15 @@ std::istream &is = *Stream;
           std::streampos end = is.tellg();
           VL l = (VL)(end - start);
           is.seekg( start, std::ios::beg );
-          F->GetDataSet().ReadWithLength<ImplicitDataElement,SwapperNoOp>(is, l);
+          //F->GetDataSet().ReadUpToTagWithLength<ImplicitDataElement,SwapperNoOp>(is, tag, l);
+          caller.template ReadCommonWithLength<ImplicitDataElement,SwapperNoOp>(is,l);
           is.peek();
           }
         }
       else
         {
-        F->GetDataSet().Read<ExplicitDataElement,SwapperNoOp>(is);
+        //F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
+        caller.template ReadCommon<ExplicitDataElement,SwapperNoOp>(is);
         }
       }
     }
@@ -365,7 +467,8 @@ std::istream &is = *Stream;
       // GDCM 1.X
       gdcmWarningMacro( "Attempt to read non CP 246" );
       F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-      F->GetDataSet().Read<CP246ExplicitDataElement,SwapperNoOp>(is);
+      //F->GetDataSet().ReadUpToTag<CP246ExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
+      caller.template ReadCommon<CP246ExplicitDataElement,SwapperNoOp>(is);
       }
     else if( ex.GetLastElement().GetVR() == VR::UN )
       {
@@ -390,7 +493,8 @@ std::istream &is = *Stream;
       // GDCM 1.X
       gdcmWarningMacro( "Attempt to read GDCM 1.X wrongly encoded");
       F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-      F->GetDataSet().Read<UNExplicitDataElement,SwapperNoOp>(is);
+      //F->GetDataSet().ReadUpToTag<UNExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
+      caller.template ReadCommon<UNExplicitDataElement,SwapperNoOp>(is);
       // This file can only be rewritten as implicit...
       }
     else if ( ex.GetLastElement().GetTag() == Tag(0xfeff,0x00e0) )
@@ -431,13 +535,14 @@ std::istream &is = *Stream;
           // seek back tag + vr:
           is.seekg( -6, std::ios::cur );
           VR16ExplicitDataElement ide;
-          ide.Read<SwapperNoOp>( is );
+          ide.template Read<SwapperNoOp>( is );
           // If we are here it means we succeeded in reading the implicit data element:
           F->GetDataSet().Insert( ide );
-          F->GetDataSet().Read<VR16ExplicitDataElement,SwapperNoOp>(is);
+          //F->GetDataSet().template Read<VR16ExplicitDataElement,SwapperNoOp>(is);
+          caller.template ReadCommon<VR16ExplicitDataElement,SwapperNoOp>(is);
           // This file can only be rewritten as implicit...
           }
-        catch ( Exception &ex )
+        catch ( Exception &ex1 )
           {
           try
             {
@@ -467,7 +572,8 @@ std::istream &is = *Stream;
             // be inverted and all would be perfect...
             gdcmWarningMacro( "Attempt to read file with explicit/implicit" );
             F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-            F->GetDataSet().Read<ExplicitImplicitDataElement,SwapperNoOp>(is);
+            //F->GetDataSet().template Read<ExplicitImplicitDataElement,SwapperNoOp>(is);
+            caller.template ReadCommon<ExplicitImplicitDataElement,SwapperNoOp>(is);
             }
           catch ( Exception &ex )
             {
@@ -492,7 +598,8 @@ std::istream &is = *Stream;
             // Explicit/Implicit
             gdcmWarningMacro( "Attempt to read file with explicit/implicit" );
             F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-            F->GetDataSet().Read<UNExplicitImplicitDataElement,SwapperNoOp>(is);
+            //F->GetDataSet().template Read<UNExplicitImplicitDataElement,SwapperNoOp>(is);
+            caller.template ReadCommon<UNExplicitImplicitDataElement,SwapperNoOp>(is);
             }
           }
         }
@@ -523,7 +630,8 @@ std::istream &is = *Stream;
 
         // Philips
         F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-        F->GetDataSet().Read<ExplicitImplicitDataElement,SwapperNoOp>(is);
+        //F->GetDataSet().ReadUpToTag<ExplicitImplicitDataElement,SwapperNoOp>(is,tag, skiptags);
+        caller.template ReadCommon<ExplicitImplicitDataElement,SwapperNoOp>(is);
         // This file can only be rewritten as implicit...
         }
       }
@@ -543,7 +651,8 @@ std::istream &is = *Stream;
     success = false;
     }
 
-    if( success ) assert( Stream->eof() );
+    //if( success ) assert( Stream->eof() );
+    caller.Check(success, *Stream );
     }
   catch( Exception &ex )
     {
@@ -573,281 +682,4 @@ std::istream &is = *Stream;
   return success;
 }
 
-bool Reader::ReadUpToTag(const Tag & tag, std::set<Tag> const & skiptags)
-{
-  if( !Stream )
-    {
-    gdcmErrorMacro( "No File" );
-    return false;
-    }
-  bool success = true;
-
-  try
-    {
-std::istream &is = *Stream;
-
-  bool haspreamble = true;
-  try
-    {
-    F->GetHeader().GetPreamble().Read( is );
-    }
-  catch( std::exception &ex )
-    {
-    // return to beginning of file, hopefully this file is simply missing preamble
-    is.seekg(0, std::ios::beg);
-    haspreamble = false;
-    }
-  catch( ... )
-    {
-    assert(0);
-    }
-
-  bool hasmetaheader = true;
-  try
-    {
-    if( haspreamble )
-      {
-      try
-        {
-        F->GetHeader().Read( is );
-        }
-      catch( std::exception &ex )
-        {
-        // Weird implicit meta header:
-        is.seekg(128+4, std::ios::beg );
-        try
-          {
-          F->GetHeader().ReadCompat(is);
-          }
-        catch( std::exception &ex )
-          {
-          // Ok I get it now... there is absolutely no meta header, giving up
-          hasmetaheader = false;
-          }
-        }
-      }
-    else
-      F->GetHeader().ReadCompat(is);
-    }
-  catch( std::exception &ex )
-    {
-    // Same player play again:
-    is.seekg(0, std::ios::beg );
-    hasmetaheader = false;
-    }
-  catch( ... )
-    {
-    // Ooops..
-    assert(0);
-    }
-
-  const TransferSyntax &ts = F->GetHeader().GetDataSetTransferSyntax();
-  if( !ts.IsValid() )
-    {
-    throw Exception( "Meta Header issue" );
-    }
-
-  //std::cerr << ts.GetNegociatedType() << std::endl;
-  //std::cerr << TransferSyntax::GetTSString(ts) << std::endl;
-  // Special case where the dataset was compressed using the deflate
-  // algorithm
-  if( ts == TransferSyntax::DeflatedExplicitVRLittleEndian )
-    {
-    zlib_stream::zip_istream gzis( is );
-    // FIXME: we also know in this case that we are dealing with Explicit:
-    assert( ts.GetNegociatedType() == TransferSyntax::Explicit );
-    F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperNoOp>(gzis,tag, skiptags);
-
-    return is;
-    }
-
-  try
-    {
-    if( ts.GetSwapCode() == SwapCode::BigEndian )
-      {
-      //US-RGB-8-epicard.dcm is big endian
-      if( ts.GetNegociatedType() == TransferSyntax::Implicit )
-        {
-        // There is no such thing as Implicit Big Endian... oh well
-        // LIBIDO-16-ACR_NEMA-Volume.dcm
-        F->GetDataSet().ReadUpToTag<ImplicitDataElement,SwapperDoOp>(is,tag, skiptags);
-        }
-      else
-        {
-        F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperDoOp>(is,tag, skiptags);
-        }
-      }
-    else // LittleEndian
-      {
-      if( ts.GetNegociatedType() == TransferSyntax::Implicit )
-        {
-        if( hasmetaheader && haspreamble )
-          {
-          F->GetDataSet().ReadUpToTag<ImplicitDataElement,SwapperNoOp>(is,tag, skiptags);
-          }
-        else
-          {
-          std::streampos start = is.tellg();
-          is.seekg( 0, std::ios::end);
-          std::streampos end = is.tellg();
-          VL l = (VL)(end - start);
-          is.seekg( start, std::ios::beg );
-          F->GetDataSet().ReadUpToTagWithLength<ImplicitDataElement,SwapperNoOp>(is, tag, l);
-          is.peek();
-          }
-        }
-      else
-        {
-        F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
-        }
-      }
-    }
-  // Only catch parse exception at this point
-  catch( ParseException &ex )
-    {
-#ifdef GDCM_SUPPORT_BROKEN_IMPLEMENTATION
-    if( ex.GetLastElement().GetVR() == VR::UN && ex.GetLastElement().IsUndefinedLength() )
-      {
-      // non CP 246
-      // P.Read( is );
-      is.clear();
-      if( haspreamble )
-        {
-        is.seekg(128+4, std::ios::beg);
-        }
-      else
-        {
-        is.seekg(0, std::ios::beg);
-        }
-      if( hasmetaheader )
-        {
-        // FIXME: we are reading twice the same meta-header, we succedeed the first time...
-        // We should be able to seek to proper place instead of re-reading
-        FileMetaInformation header;
-        header.Read(is);
-        }
-
-      // GDCM 1.X
-      gdcmWarningMacro( "Attempt to read non CP 246" );
-      F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-      F->GetDataSet().ReadUpToTag<CP246ExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
-      }
-    else if( ex.GetLastElement().GetVR() == VR::UN )
-      {
-      // P.Read( is );
-      is.clear();
-      if( haspreamble )
-        {
-        is.seekg(128+4, std::ios::beg);
-        }
-      else
-        {
-        is.seekg(0, std::ios::beg);
-        }
-      if( hasmetaheader )
-        {
-        // FIXME: we are reading twice the same meta-header, we succedeed the first time...
-        // We should be able to seek to proper place instead of re-reading
-        FileMetaInformation header;
-        header.Read(is);
-        }
-
-      // GDCM 1.X
-      gdcmWarningMacro( "Attempt to read GDCM 1.X wrongly encoded");
-      F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-      F->GetDataSet().ReadUpToTag<UNExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
-      // This file can only be rewritten as implicit...
-      }
-    else if ( ex.GetLastElement().GetTag() == Tag(0xfeff,0x00e0) )
-      {
-      // Famous philips where some private sequence were byteswapped !
-      // eg. PHILIPS_Intera-16-MONO2-Uncompress.dcm
-      // P.Read( is );
-      is.clear();
-      if( haspreamble )
-        {
-        is.seekg(128+4, std::ios::beg);
-        }
-      else
-        {
-        is.seekg(0, std::ios::beg);
-        }
-      if( hasmetaheader )
-        {
-        // FIXME: we are reading twice the same meta-header, we succedeed the first time...
-        // We should be able to seek to proper place instead of re-reading
-        FileMetaInformation header;
-        header.Read(is);
-        }
-
-      //
-      gdcmWarningMacro( "Attempt to read Philips with ByteSwap private sequence wrongly encoded");
-      F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-      assert(0);  // TODO FIXME
-      }
-    else
-      {
-      // Let's try again with an ExplicitImplicitDataElement:
-      if( ts.GetSwapCode() == SwapCode::LittleEndian &&
-        ts.GetNegociatedType() == TransferSyntax::Explicit )
-        {
-        // P.Read( is );
-        if( haspreamble )
-          {
-          is.seekg(128+4, std::ios::beg);
-          }
-        else
-          {
-          is.seekg(0, std::ios::beg);
-          }
-        if( hasmetaheader )
-          {
-          // FIXME: we are reading twice the same meta-header, we succedeed the first time...
-          // We should be able to seek to proper place instead of re-reading
-          FileMetaInformation header;
-          header.ReadCompat(is);
-          }
-
-        // Philips
-        gdcmWarningMacro( "Attempt to read the file as mixture of explicit/implicit");
-        F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-        F->GetDataSet().ReadUpToTag<ExplicitImplicitDataElement,SwapperNoOp>(is,tag, skiptags);
-        // This file can only be rewritten as implicit...
-        }
-      }
-#else
-    std::cerr << ex.what() << std::endl;
-    success = false;
-#endif /* GDCM_SUPPORT_BROKEN_IMPLEMENTATION */
-    }
-  catch( Exception &ex )
-    {
-    gdcmDebugMacro( ex.what() );
-    success = false;
-    }
-  catch( ... )
-    {
-    gdcmWarningMacro( "Unknown exception" );
-    success = false;
-    }
-
-    //assert( Stream->eof() );
-    }
-  catch( Exception &ex )
-    {
-    gdcmDebugMacro( ex.what() );
-    success = false;
-    }
-  catch( ... )
-    {
-    gdcmWarningMacro( "Unknown exception" );
-    success = false;
-    }
-
-  // FIXME : call this function twice...
-  //Stream->close();
-
-  return success;
-}
-
 } // end namespace gdcm
