@@ -3,7 +3,7 @@
   Program: GDCM (Grassroots DICOM). A DICOM library
   Module:  $URL$
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -35,55 +35,55 @@ namespace gdcm
 // Table E.1-1
 // BALCPA
 static Tag BasicApplicationLevelConfidentialityProfileAttributes[] = {
-//              Attribute Name                                            Tag
-/*              Instance Creator UID                                  */ Tag(0x0008,0x0014),
-/*              SOP Instance UID                                      */ Tag(0x0008,0x0018),
-/*              Accession Number                                      */ Tag(0x0008,0x0050),
-/*              Institution Name                                      */ Tag(0x0008,0x0080),
-/*              Institution Address                                   */ Tag(0x0008,0x0081),
-/*              Referring Physician's Name                            */ Tag(0x0008,0x0090),
-/*              Referring Physician's Address                         */ Tag(0x0008,0x0092),
-/*              Referring Physician's Telephone Numbers               */ Tag(0x0008,0x0094),
-/*              Station Name                                          */ Tag(0x0008,0x1010),
-/*              Study Description                                     */ Tag(0x0008,0x1030),
-/*              Series Description                                    */ Tag(0x0008,0x103E),
-/*              Institutional Department Name                         */ Tag(0x0008,0x1040),
-/*              Physician(s) of Record                                */ Tag(0x0008,0x1048),
-/*              Performing Physicians' Name                           */ Tag(0x0008,0x1050),
-/*              Name of Physician(s) Reading Study                    */ Tag(0x0008,0x1060),
-/*              Operators' Name                                       */ Tag(0x0008,0x1070),
-/*              Admitting Diagnoses Description                       */ Tag(0x0008,0x1080),
-/*              Referenced SOP Instance UID                           */ Tag(0x0008,0x1155),
-/*              Derivation Description                                */ Tag(0x0008,0x2111),
-/*              Patient's Name                                        */ Tag(0x0010,0x0010),
-/*              Patient ID                                            */ Tag(0x0010,0x0020),
-/*              Patient's Birth Date                                  */ Tag(0x0010,0x0030),
-/*              Patient's Birth Time                                  */ Tag(0x0010,0x0032),
-/*              Patient's Sex                                         */ Tag(0x0010,0x0040),
-/*              Other Patient Ids                                     */ Tag(0x0010,0x1000),
-/*              Other Patient Names                                   */ Tag(0x0010,0x1001),
-/*              Patient's Age                                         */ Tag(0x0010,0x1010),
-/*              Patient's Size                                        */ Tag(0x0010,0x1020),
-/*              Patient's Weight                                      */ Tag(0x0010,0x1030),
-/*              Medical Record Locator                                */ Tag(0x0010,0x1090),
-/*              Ethnic Group                                          */ Tag(0x0010,0x2160),
-/*              Occupation                                            */ Tag(0x0010,0x2180),
-/*              Additional Patient's History                          */ Tag(0x0010,0x21B0),
-/*              Patient Comments                                      */ Tag(0x0010,0x4000),
-/*              Device Serial Number                                  */ Tag(0x0018,0x1000),
-/*              Protocol Name                                         */ Tag(0x0018,0x1030),
-/*              Study Instance UID                                    */ Tag(0x0020,0x000D),
-/*              Series Instance UID                                   */ Tag(0x0020,0x000E),
-/*              Study ID                                              */ Tag(0x0020,0x0010),
-/*              Frame of Reference UID                                */ Tag(0x0020,0x0052),
-/*              Synchronization Frame of Reference UID                */ Tag(0x0020,0x0200),
-/*              Image Comments                                        */ Tag(0x0020,0x4000),
-/*              Request Attributes Sequence                           */ Tag(0x0040,0x0275),
-/*              UID                                                   */ Tag(0x0040,0xA124),
-/*              Content Sequence                                      */ Tag(0x0040,0xA730),
-/*              Storage Media File-set UID                            */ Tag(0x0088,0x0140),
-/*              Referenced Frame of Reference UID                     */ Tag(0x3006,0x0024),
-/*              Related Frame of Reference UID                        */ Tag(0x3006,0x00C2)
+//    Attribute Name                                Tag
+/*    Instance Creator UID                      */ Tag(0x0008,0x0014),
+/*    SOP Instance UID                          */ Tag(0x0008,0x0018),
+/*    Accession Number                          */ Tag(0x0008,0x0050),
+/*    Institution Name                          */ Tag(0x0008,0x0080),
+/*    Institution Address                       */ Tag(0x0008,0x0081),
+/*    Referring Physician's Name                */ Tag(0x0008,0x0090),
+/*    Referring Physician's Address             */ Tag(0x0008,0x0092),
+/*    Referring Physician's Telephone Numbers   */ Tag(0x0008,0x0094),
+/*    Station Name                              */ Tag(0x0008,0x1010),
+/*    Study Description                         */ Tag(0x0008,0x1030),
+/*    Series Description                        */ Tag(0x0008,0x103E),
+/*    Institutional Department Name             */ Tag(0x0008,0x1040),
+/*    Physician(s) of Record                    */ Tag(0x0008,0x1048),
+/*    Performing Physicians' Name               */ Tag(0x0008,0x1050),
+/*    Name of Physician(s) Reading Study        */ Tag(0x0008,0x1060),
+/*    Operators' Name                           */ Tag(0x0008,0x1070),
+/*    Admitting Diagnoses Description           */ Tag(0x0008,0x1080),
+/*    Referenced SOP Instance UID               */ Tag(0x0008,0x1155),
+/*    Derivation Description                    */ Tag(0x0008,0x2111),
+/*    Patient's Name                            */ Tag(0x0010,0x0010),
+/*    Patient ID                                */ Tag(0x0010,0x0020),
+/*    Patient's Birth Date                      */ Tag(0x0010,0x0030),
+/*    Patient's Birth Time                      */ Tag(0x0010,0x0032),
+/*    Patient's Sex                             */ Tag(0x0010,0x0040),
+/*    Other Patient Ids                         */ Tag(0x0010,0x1000),
+/*    Other Patient Names                       */ Tag(0x0010,0x1001),
+/*    Patient's Age                             */ Tag(0x0010,0x1010),
+/*    Patient's Size                            */ Tag(0x0010,0x1020),
+/*    Patient's Weight                          */ Tag(0x0010,0x1030),
+/*    Medical Record Locator                    */ Tag(0x0010,0x1090),
+/*    Ethnic Group                              */ Tag(0x0010,0x2160),
+/*    Occupation                                */ Tag(0x0010,0x2180),
+/*    Additional Patient's History              */ Tag(0x0010,0x21B0),
+/*    Patient Comments                          */ Tag(0x0010,0x4000),
+/*    Device Serial Number                      */ Tag(0x0018,0x1000),
+/*    Protocol Name                             */ Tag(0x0018,0x1030),
+/*    Study Instance UID                        */ Tag(0x0020,0x000D),
+/*    Series Instance UID                       */ Tag(0x0020,0x000E),
+/*    Study ID                                  */ Tag(0x0020,0x0010),
+/*    Frame of Reference UID                    */ Tag(0x0020,0x0052),
+/*    Synchronization Frame of Reference UID    */ Tag(0x0020,0x0200),
+/*    Image Comments                            */ Tag(0x0020,0x4000),
+/*    Request Attributes Sequence               */ Tag(0x0040,0x0275),
+/*    UID                                       */ Tag(0x0040,0xA124),
+/*    Content Sequence                          */ Tag(0x0040,0xA730),
+/*    Storage Media File-set UID                */ Tag(0x0088,0x0140),
+/*    Referenced Frame of Reference UID         */ Tag(0x3006,0x0024),
+/*    Related Frame of Reference UID            */ Tag(0x3006,0x00C2)
 };
 
 
@@ -93,6 +93,7 @@ Anonymizer::~Anonymizer()
 
 bool Anonymizer::Empty( Tag const &t)
 {
+  // There is a secret code path to make it work for VR::SQ since operation is just 'make empty'
   return Replace(t, "", 0);
 }
 
@@ -131,7 +132,11 @@ bool Anonymizer::Replace( Tag const &t, const char *value, VL const & vl )
         DataElement de ( ds.GetDataElement(t) );
         if ( de.GetVR() != VR::INVALID )
           {
-          assert( de.GetVR() != VR::SQ );
+          if( de.GetVR() == VR::SQ )
+            {
+            gdcmDebugMacro( "Cannot replace a VR:SQ" );
+            return false;
+            }
           }
         de.SetByteValue( "", vl );
         ds.Insert( de );
@@ -155,8 +160,19 @@ bool Anonymizer::Replace( Tag const &t, const char *value, VL const & vl )
       || dictentry.GetVR() == VR::SQ
     )
       {
-      // Let's give up !
-      gdcmWarningMacro( "Cannot process tag: " << t << " with vr: " << dictentry.GetVR() );
+      // Make the VR::SQ empty
+      if( dictentry.GetVR() == VR::SQ && vl == 0 && value && *value == 0 )
+        {
+        DataElement de( t );
+        de.SetVR( VR::SQ );
+        //de.SetByteValue( "", 0 );
+        ds.Replace( de );
+        }
+      else
+        {
+        // Let's give up !
+        gdcmWarningMacro( "Cannot process tag: " << t << " with vr: " << dictentry.GetVR() );
+        }
       //ret = false;
       }
     else if ( dictentry.GetVR() & VR::VRBINARY )
@@ -203,118 +219,184 @@ bool Anonymizer::Replace( Tag const &t, const char *value, VL const & vl )
       {
       // vr from dict seems to be ascii, so it seems resonable to write a ByteValue here:
       assert( dictentry.GetVR() & VR::VRASCII );
-      assert( value );
-      std::string padded( value, vl );
-      // All ASCII VR needs to be padded with a space
-      if( vl.IsOdd() )
+      if( value )
         {
-        if( dictentry.GetVR() == VR::UI )
+        std::string padded( value, vl );
+        // All ASCII VR needs to be padded with a space
+        if( vl.IsOdd() )
           {
-          // \0 is automatically added when using a ByteValue
+          if( dictentry.GetVR() == VR::UI )
+            {
+            // \0 is automatically added when using a ByteValue
+            }
+          else
+            {
+            padded += " ";
+            }
+          }
+        // Hum, we could have cases where a public element would not be known, in which case
+        // it is a good idea to first check for the VR as found in the file:
+        DataElement de( t );
+        if( ds.FindDataElement( t ) )
+          {
+          de.SetVR( ds.GetDataElement(t).GetVR() );
           }
         else
           {
-          padded += " ";
+          de.SetVR( dictentry.GetVR() );
           }
+        de.SetByteValue( padded.c_str(), padded.size() );
+        ds.Replace( de );
+        ret = true;
         }
-      // Hum, we could have cases where a public element would not be known, in which case
-      // it is a good idea to first check for the VR as found in the file:
-      DataElement de( t );
-      if( ds.FindDataElement( t ) )
-        {
-        de.SetVR( ds.GetDataElement(t).GetVR() );
-        }
-      else
-        {
-        de.SetVR( dictentry.GetVR() );
-        }
-      de.SetByteValue( padded.c_str(), padded.size() );
-      ds.Replace( de );
-      ret = true;
       }
     }
   return ret;
 }
 
-bool Anonymizer::RemoveRetired()
+static bool Anonymizer_RemoveRetired(File const &file, DataSet &ds)
 {
   static const Global &g = GlobalInstance;
   static const Dicts &dicts = g.GetDicts();
   static const Dict &pubdict = dicts.GetPublicDict();
-
-  DataSet &ds = F->GetDataSet();
   DataSet::Iterator it = ds.Begin();
   for( ; it != ds.End(); )
     {
-    const DataElement &de = *it;
-    if( de.GetTag().IsPublic() )
+    const DataElement &de1 = *it;
+    // std::set::erase invalidate iterator, so we need to make a copy first:
+    DataSet::Iterator dup = it;
+    ++it;
+    if( de1.GetTag().IsPublic() )
       {
-      const DictEntry &entry = pubdict.GetDictEntry( de.GetTag() );
+      const DictEntry &entry = pubdict.GetDictEntry( de1.GetTag() );
       if( entry.GetRetired() )
         {
-        // std::set::erase invalidate iterator, so we need to make a copy first:
-        DataSet::Iterator dup = it;
-        ++it;
         ds.GetDES().erase(dup);
         }
-      else
-        {
-      ++it;
-        }
       }
     else
       {
-      ++it;
+      const DataElement &de = *dup;
+      VR vr = DataSetHelper::ComputeVR(file, ds, de.GetTag() );
+      if( vr.Compatible(VR::SQ) )
+        {
+        SmartPointer<SequenceOfItems> sq = de.GetValueAsSQ();
+        if( sq )
+          {
+          unsigned int n = sq->GetNumberOfItems();
+          for( unsigned int i = 1; i <= n; i++) // item starts at 1, not 0
+            {
+            Item &item = sq->GetItem( i );
+            DataSet &nested = item.GetNestedDataSet();
+            Anonymizer_RemoveRetired( file, nested );
+            }
+          DataElement de_dup = *dup;
+          de_dup.SetValue( *sq );
+          de_dup.SetVLToUndefined(); // FIXME
+          ds.Replace( de_dup );
+          }
+        }
       }
     }
+  return true;
+}
 
+bool Anonymizer::RemoveRetired()
+{
+  DataSet &ds = F->GetDataSet();
+  return Anonymizer_RemoveRetired(*F, ds);
+}
+
+static bool Anonymizer_RemoveGroupLength(File const &file, DataSet &ds)
+{
+  DataSet::Iterator it = ds.Begin();
+  for( ; it != ds.End(); )
+    {
+    const DataElement &de1 = *it;
+    // std::set::erase invalidate iterator, so we need to make a copy first:
+    DataSet::Iterator dup = it;
+    ++it;
+    if( de1.GetTag().IsGroupLength() )
+      {
+      ds.GetDES().erase(dup);
+      }
+    else
+      {
+      const DataElement &de = *dup;
+      VR vr = DataSetHelper::ComputeVR(file, ds, de.GetTag() );
+      if( vr.Compatible(VR::SQ) )
+        {
+        SmartPointer<SequenceOfItems> sq = de.GetValueAsSQ();
+        if( sq )
+          {
+          unsigned int n = sq->GetNumberOfItems();
+          for( unsigned int i = 1; i <= n; i++) // item starts at 1, not 0
+            {
+            Item &item = sq->GetItem( i );
+            DataSet &nested = item.GetNestedDataSet();
+            Anonymizer_RemoveGroupLength( file, nested );
+            }
+          DataElement de_dup = *dup;
+          de_dup.SetValue( *sq );
+          de_dup.SetVLToUndefined(); // FIXME
+          ds.Replace( de_dup );
+          }
+        }
+      }
+    }
   return true;
 }
 
 bool Anonymizer::RemoveGroupLength()
 {
   DataSet &ds = F->GetDataSet();
+  return Anonymizer_RemoveGroupLength(*F, ds);
+}
+
+static bool Anonymizer_RemovePrivateTags(File const &file, DataSet &ds)
+{
   DataSet::Iterator it = ds.Begin();
   for( ; it != ds.End(); )
     {
-    const DataElement &de = *it;
-    if( de.GetTag().IsGroupLength() )
-      {
+    const DataElement &de1 = *it;
       // std::set::erase invalidate iterator, so we need to make a copy first:
       DataSet::Iterator dup = it;
       ++it;
+    if( de1.GetTag().IsPrivate() )
+      {
       ds.GetDES().erase(dup);
       }
     else
       {
-      ++it;
+      const DataElement &de = *dup;
+      VR vr = DataSetHelper::ComputeVR(file, ds, de.GetTag() );
+      if( vr.Compatible(VR::SQ) )
+        {
+        SmartPointer<SequenceOfItems> sq = de.GetValueAsSQ();
+        if( sq )
+          {
+          unsigned int n = sq->GetNumberOfItems();
+          for( unsigned int i = 1; i <= n; i++) // item starts at 1, not 0
+            {
+            Item &item = sq->GetItem( i );
+            DataSet &nested = item.GetNestedDataSet();
+            Anonymizer_RemovePrivateTags( file, nested );
+            }
+          DataElement de_dup = *dup;
+          de_dup.SetValue( *sq );
+          de_dup.SetVLToUndefined(); // FIXME
+          ds.Replace( de_dup );
+          }
+        }
       }
     }
-
   return true;
 }
 
 bool Anonymizer::RemovePrivateTags()
 {
   DataSet &ds = F->GetDataSet();
-  DataSet::Iterator it = ds.Begin();
-  for( ; it != ds.End(); )
-    {
-    const DataElement &de = *it;
-    if( de.GetTag().IsPrivate() )
-      {
-      // std::set::erase invalidate iterator, so we need to make a copy first:
-      DataSet::Iterator dup = it;
-      ++it;
-      ds.GetDES().erase(dup);
-      }
-    else
-      {
-      ++it;
-      }
-    }
-
-  return true;
+  return Anonymizer_RemovePrivateTags(*F, ds);
 }
 
 /*
@@ -338,6 +420,57 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile(bool deidentify)
   return ret;
 }
 
+std::vector<Tag> Anonymizer::GetBasicApplicationLevelConfidentialityProfileAttributes()
+{
+  static const unsigned int deidSize = sizeof(Tag);
+  static const unsigned int numDeIds = sizeof(BasicApplicationLevelConfidentialityProfileAttributes) / deidSize;
+  static const Tag *start = BasicApplicationLevelConfidentialityProfileAttributes;
+  static const Tag *end = start + numDeIds;
+  return std::vector<Tag>(start, end);
+}
+
+bool Anonymizer::CheckIfSequenceContainsAttributeToAnonymize(File const &file, SequenceOfItems* sqi) const
+{
+  static const unsigned int deidSize = sizeof(Tag);
+  static const unsigned int numDeIds = sizeof(BasicApplicationLevelConfidentialityProfileAttributes) / deidSize;
+  static const Tag *start = BasicApplicationLevelConfidentialityProfileAttributes;
+  static const Tag *end = start + numDeIds;
+
+  bool found = false;
+  for(const Tag *ptr = start ; ptr != end && !found ; ++ptr)
+    {
+    const Tag& tag = *ptr;
+    found = sqi->FindDataElement( tag );
+    }
+  // ok we can exit.
+  if( found ) return true;
+
+  // now look into sub-sequence:
+  unsigned int n = sqi->GetNumberOfItems();
+  for( unsigned int i = 1; i <= n; i++) // item starts at 1, not 0
+    {
+    Item &item = sqi->GetItem( i );
+    DataSet &nested = item.GetNestedDataSet();
+    DataSet::Iterator it = nested.Begin();
+    for( ; it != nested.End() && !found; ++it)
+      {
+      const DataElement &de = *it;
+      VR vr = DataSetHelper::ComputeVR(file, nested, de.GetTag() );
+      SmartPointer<SequenceOfItems> sqi = 0;
+      if( vr == VR::SQ )
+        {
+        sqi = de.GetValueAsSQ();
+        }
+      if( sqi )
+        {
+        found = CheckIfSequenceContainsAttributeToAnonymize(file, sqi);
+        }
+      }
+    }
+
+  return found;
+}
+
 // Implementation note:
 // This function trigger:
 // 1 StartEvent
@@ -355,6 +488,13 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
   //p7.SetCertificate( this->x509 );
 
   DataSet &ds = F->GetDataSet();
+  if(  ds.FindDataElement( Tag(0x0400,0x0500) ) 
+    || ds.FindDataElement( Tag(0x0012,0x0062) ) 
+    || ds.FindDataElement( Tag(0x0012,0x0063) ) )
+    {
+    gdcmDebugMacro( "EncryptedContentTransferSyntax Attribute is present !" );
+    return false;
+    }
 
   // PS 3.15
   // E.1 BASIC APPLICATION LEVEL CONFIDENTIALITY PROFILE
@@ -374,8 +514,8 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
   // might have been created. Only a single Item shall be present.
 
   // Create a Sequence
-  SmartPointer<SequenceOfItems> sq = new SequenceOfItems();
-  sq->SetLengthToUndefined();
+  SmartPointer<SequenceOfItems> sq1 = new SequenceOfItems();
+  sq1->SetLengthToUndefined();
 
   // Create a *single* item
   Item it;
@@ -396,7 +536,6 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
     {
     const DataElement &de = *it;
     //const SequenceOfItems *sqi = de.GetSequenceOfItems();
-    const ByteValue *bv = de.GetByteValue();
     SmartPointer<SequenceOfItems> sqi = 0;
     VR vr = DataSetHelper::ComputeVR(*F, ds, de.GetTag() );
     if( vr == VR::SQ )
@@ -405,27 +544,32 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
       }
     if( sqi )
       {
-      bool found = false;
-      for(const Tag *ptr = start ; ptr != end && !found ; ++ptr)
-        {
-        const Tag& tag = *ptr;
-        found = sqi->FindDataElement( tag );
-        }
+      bool found
+        = CheckIfSequenceContainsAttributeToAnonymize(*F, sqi);
       if( found )
         {
-        // A special Tag was found let's store the entire Sequence of Item:
-        encryptedds.Insert( de );
+        // A special Tag was found within the SQ,let's store the entire Sequence of Item:
+        if( !encryptedds.FindDataElement( de.GetTag() ) )
+          {
+          // What if we found a Patient Name within a Content Sequence
+          // we do not need to insert twice this DICOM Attribute
+          encryptedds.Insert( de );
+          }
+        else
+          {
+          assert( de == encryptedds.GetDataElement( de.GetTag() ) );
+          }
         }
       }
     }
 }
 
   this->InvokeEvent( IterationEvent() );
-  sq->AddItem(it);
+  sq1->AddItem(it);
 
   DataElement des( Tag(0x0400,0x0550) );
   des.SetVR(VR::SQ);
-  des.SetValue(*sq);
+  des.SetValue(*sq1);
   des.SetVLToUndefined();
 
   std::ostringstream os;
@@ -446,6 +590,11 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
 
   size_t encrypted_len2 = encrypted_len;
   bool b = p7.Encrypt( buf, encrypted_len, orig, encrypted_str.size() );
+  if( !b ) 
+  {
+    gdcmErrorMacro( "Problem with Encrypt" );
+    return false;
+  }
   assert( encrypted_len <= encrypted_len2 );
 
     {
@@ -454,7 +603,8 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
     sq->SetLengthToUndefined();
 
     // FIXME: should be user configurable:
-    TransferSyntax encrypted_ts = TransferSyntax::ImplicitVRLittleEndian;
+    //TransferSyntax encrypted_ts = TransferSyntax::ImplicitVRLittleEndian;
+    TransferSyntax encrypted_ts = TransferSyntax::ExplicitVRLittleEndian;
     // <entry group="0400" element="0510" vr="UI" vm="1" name="Encrypted Content Transfer Syntax UID"/>
     DataElement encrypted_ts_de( Tag(0x400,0x510) );
     encrypted_ts_de.SetVR( Attribute<0x0400, 0x0510>::GetVR() );
@@ -492,12 +642,12 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
     // be used by unauthorized entities to identify the patient.
 
     // Insert sequence into data set
-    DataElement des( Tag(0x0400,0x0500) );
-    des.SetVR(VR::SQ);
-    des.SetValue(*sq);
-    des.SetVLToUndefined();
+    DataElement subdes( Tag(0x0400,0x0500) );
+    subdes.SetVR(VR::SQ);
+    subdes.SetValue(*sq);
+    subdes.SetVLToUndefined();
 
-    ds.Insert(des);
+    ds.Insert(subdes);
     }
   this->InvokeEvent( IterationEvent() );
 
@@ -511,7 +661,20 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
   //  if( ds.FindDataElement( tag ) ) BALCPProtect(F->GetDataSet(), tag);
   //  }
   // Check that root level sequence do not contains any of those attributes
-  RecurseDataSet( F->GetDataSet() );
+  try
+  {
+    RecurseDataSet( F->GetDataSet() );
+  }
+  catch(std::exception &ex)
+  {
+  gdcmDebugMacro( "Problem during RecurseDataSet" );
+  return false;
+  }
+catch(...)
+{
+  gdcmDebugMacro( "Unknown Problem during RecurseDataSet" );
+  return false;
+}
 
   this->InvokeEvent( IterationEvent() );
 
@@ -543,7 +706,7 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
 
 bool IsVRUI(Tag const &tag)
 {
-  static const Global &g = GlobalInstance;
+  static const Global &g = Global::GetInstance();
   static const Dicts &dicts = g.GetDicts();
   const DictEntry &dictentry = dicts.GetDictEntry(tag);
   if( dictentry.GetVR() == VR::UI ) return true;
@@ -557,19 +720,22 @@ bool IsVRUI(Tag const &tag)
 }
 
 static const Tag SpecialTypeTags[] = {
-/*              Patient's Name                                        */ Tag(0x0010,0x0010),
-/*              Patient ID                                            */ Tag(0x0010,0x0020),
-/*              Study ID                                              */ Tag(0x0020,0x0010),
-/*              Series Number                                         */ Tag(0x0020,0x0011)
+/*   Patient's Name          */ Tag(0x0010,0x0010),
+/*   Patient ID              */ Tag(0x0010,0x0020),
+/*   Study ID                */ Tag(0x0020,0x0010),
+/*   Series Number           */ Tag(0x0020,0x0011)
 };
 
-bool Anonymizer::CanEmptyTag(Tag const &tag)
+bool Anonymizer::CanEmptyTag(Tag const &tag, const IOD &iod) const
 {
-  static const Global &g = GlobalInstance;
-  static const Dicts &dicts = g.GetDicts();
+  static const Global &g = Global::GetInstance();
+  //static const Dicts &dicts = g.GetDicts();
   static const Defs &defs = g.GetDefs();
-  DataSet &ds = F->GetDataSet();
-  Type t = defs.GetTypeFromTag(*F, tag);
+  const DataSet &ds = F->GetDataSet(); (void)ds;
+  //Type told = defs.GetTypeFromTag(*F, tag);
+  Type t = iod.GetTypeFromTag(defs, tag);
+  //assert( t == told );
+
   gdcmDebugMacro( "Type for tag=" << tag << " is " << t );
 
   //assert( t != Type::UNKNOWN );
@@ -616,12 +782,12 @@ generate a DICOMDIR
   return !b;
 }
 
-bool Anonymizer::BALCPProtect(DataSet &ds, Tag const & tag)
+bool Anonymizer::BALCPProtect(DataSet &ds, Tag const & tag, IOD const & iod)
 {
   // \precondition
   assert( ds.FindDataElement(tag) );
 
-  gdcm::AnonymizeEvent ae;
+  AnonymizeEvent ae;
   ae.SetTag( tag );
   this->InvokeEvent( ae );
 
@@ -632,12 +798,12 @@ bool Anonymizer::BALCPProtect(DataSet &ds, Tag const & tag)
 
   //DataSet &ds = F->GetDataSet();
 
-  bool canempty = CanEmptyTag( tag );
-  DataElement copy;
+  bool canempty = CanEmptyTag( tag, iod );
   if( !canempty )
     {
     TagValueKey tvk;
     tvk.first = tag;
+    DataElement copy;
     copy = ds.GetDataElement( tag );
     // gdcmData/LEADTOOLS_FLOWERS-16-MONO2-JpegLossless.dcm
     // has an empty 0008,0018 attribute, let's try to handle that:
@@ -690,13 +856,17 @@ void Anonymizer::RecurseDataSet( DataSet & ds )
   static const Tag *start = BasicApplicationLevelConfidentialityProfileAttributes;
   static const Tag *end = start + numDeIds;
 
+  static const Global &g = Global::GetInstance();
+  static const Defs &defs = g.GetDefs();
+  const IOD& iod = defs.GetIODFromFile(*F);
+
   for(const Tag *ptr = start ; ptr != end ; ++ptr)
     {
     const Tag& tag = *ptr;
     // FIXME Type 1 !
     if( ds.FindDataElement( tag ) )
       {
-      BALCPProtect(ds, tag);
+      BALCPProtect(ds, tag, iod);
       }
     }
 
@@ -714,7 +884,9 @@ void Anonymizer::RecurseDataSet( DataSet & ds )
       }
     if( sqi )
       {
+      de.SetValue( *sqi ); // EXTREMELY IMPORTANT #2912092
       de.SetVLToUndefined();
+      assert( sqi->IsUndefinedLength() );
       //de.GetVL().SetToUndefined();
       //sqi->SetLengthToUndefined();
       unsigned int n = sqi->GetNumberOfItems();
@@ -752,12 +924,42 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile2()
   //p7.SetCertificate( this->x509 );
 
   DataSet &ds = F->GetDataSet();
+  if( !ds.FindDataElement( Tag(0x0400,0x0500) ) )
+    {
+    gdcmDebugMacro( "Could not find EncryptedAttributesSQ" );
+    return false;
+    }
   const DataElement &EncryptedAttributesSequence = ds.GetDataElement( Tag(0x0400,0x0500) );
   //const SequenceOfItems *sq = EncryptedAttributesSequence.GetSequenceOfItems();
   SmartPointer<SequenceOfItems> sq = EncryptedAttributesSequence.GetValueAsSQ();
   const Item &item = sq->GetItem(1);
-  const DataSet &nds = item.GetNestedDataSet();
-  const DataElement &EncryptedContent = nds.GetDataElement( Tag(0x0400,0x0520) );
+  const DataSet &nds1 = item.GetNestedDataSet();
+  if( !nds1.FindDataElement( Tag(0x0400,0x0510) ) 
+    || nds1.GetDataElement( Tag(0x0400,0x0510) ).IsEmpty() )
+    {
+    gdcmDebugMacro( "Missing EncryptedContentTransferSyntax Attribute" );
+    return false;
+    }
+
+  const DataElement &EncryptedContentTransferSyntax = nds1.GetDataElement( Tag(0x0400,0x0510) );
+
+  std::string ts( EncryptedContentTransferSyntax.GetByteValue()->GetPointer(),
+    EncryptedContentTransferSyntax.GetByteValue()->GetLength() );
+
+  if( TransferSyntax::GetTSType( ts.c_str() ) != TransferSyntax::ExplicitVRLittleEndian )
+    {
+    gdcmDebugMacro( "Only ExplicitVRLittleEndian is supported" );
+    return false;
+    }
+
+  if( !nds1.FindDataElement( Tag(0x0400,0x0520) ) 
+    || nds1.GetDataElement( Tag(0x0400,0x0520) ).IsEmpty() )
+    {
+    gdcmDebugMacro( "Missing EncryptedContent Attribute" );
+    return false;
+    }
+
+  const DataElement &EncryptedContent = nds1.GetDataElement( Tag(0x0400,0x0520) );
   const ByteValue *bv = EncryptedContent.GetByteValue();
   
   size_t encrypted_len = bv->GetLength();
@@ -786,8 +988,8 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile2()
   //catch( std::Exception  & e)
   catch( ... )
     {
-  delete[] buf;
-  delete[] orig;
+    delete[] buf;
+    delete[] orig;
 
     return false;
     }
@@ -798,7 +1000,7 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile2()
   //std::cout << des << std::endl; 
   //std::cout << dummy << std::endl; 
   //std::cout << ss.tellg() << std::endl; 
-  assert( ss.tellg() <= encrypted_len );
+  assert( (size_t)ss.tellg() <= encrypted_len );
   // TODO: check that for i = ss.tellg() to encrypted_len, ss[i] == 0
   delete[] buf;
   delete[] orig;
@@ -812,9 +1014,9 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile2()
   SmartPointer<SequenceOfItems> sqi = dummy.GetValueAsSQ();
   assert( sqi && sqi->GetNumberOfItems() == 1 );
   Item const & item = sqi->GetItem( 1 );
-  const DataSet &nds = item.GetNestedDataSet();
-  DataSet::ConstIterator it = nds.Begin();
-  for( ; it != nds.End(); ++it )
+  const DataSet &nds2 = item.GetNestedDataSet();
+  DataSet::ConstIterator it = nds2.Begin();
+  for( ; it != nds2.End(); ++it )
     {
     ds.Replace( *it );
     }
@@ -832,7 +1034,7 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile2()
   return true;
 }
 
-void Anonymizer::SetCryptographicMessageSyntax( CryptographicMessageSyntax *cms)
+void Anonymizer::SetCryptographicMessageSyntax(CryptographicMessageSyntax *cms)
 {
   CMS = cms;
 }
