@@ -3,7 +3,7 @@
   Program: GDCM (Grassroots DICOM). A DICOM library
   Module:  $URL$
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -12,8 +12,8 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-#ifndef __gdcmCoder_h
-#define __gdcmCoder_h
+#ifndef GDCMCODER_H
+#define GDCMCODER_H
 
 #include "gdcmTypes.h"
 #include "gdcmDataElement.h" // FIXME
@@ -34,12 +34,14 @@ public:
   /// Return whether this coder support this transfer syntax (can code it)
   virtual bool CanCode(TransferSyntax const &) const = 0;
 
+  // Note: in / out are reserved keyword in C#. Change to in_ / out_
+
   /// Code
-  virtual bool Code(DataElement const &in, DataElement &out) { (void)in; (void)out; return false; }
+  virtual bool Code(DataElement const &in_, DataElement &out_) { (void)in_; (void)out_; return false; }
 protected:
-  virtual bool InternalCode(const char *bv, unsigned long len, std::ostream &os) { (void)bv;(void)os;return false; } 
+  virtual bool InternalCode(const char *bv, unsigned long len, std::ostream &os) { (void)bv;(void)os;(void)len;return false; } 
 };
 
 } // end namespace gdcm
 
-#endif //__gdcmCoder_h
+#endif //GDCMCODER_H
