@@ -3,7 +3,7 @@
   Program: GDCM (Grassroots DICOM). A DICOM library
   Module:  $URL$
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -75,7 +75,7 @@ public:
 void JPEG2000Codec::SetRate(unsigned int idx, double rate)
 {
   Internals->coder_param.tcp_rates[idx] = rate;
-  if( Internals->coder_param.tcp_numlayers <= idx )
+  if( Internals->coder_param.tcp_numlayers <= (int)idx )
     {
     Internals->coder_param.tcp_numlayers = idx + 1;
     }
@@ -90,7 +90,7 @@ double JPEG2000Codec::GetRate(unsigned int idx ) const
 void JPEG2000Codec::SetQuality(unsigned int idx, double q)
 {
   Internals->coder_param.tcp_distoratio[idx] = q;
-  if( Internals->coder_param.tcp_numlayers <= idx )
+  if( Internals->coder_param.tcp_numlayers <= (int)idx )
     {
     Internals->coder_param.tcp_numlayers = idx + 1;
     }
@@ -156,7 +156,7 @@ bool JPEG2000Codec::Decode(DataElement const &in, DataElement &out)
   if( NumberOfDimensions == 2 )
     {
     const SequenceOfFragments *sf = in.GetSequenceOfFragments();
-    assert( sf );
+    if( !sf ) return false;
     std::stringstream is;
     unsigned long totalLen = sf->ComputeByteLength();
     char *buffer = new char[totalLen];
@@ -182,9 +182,13 @@ bool JPEG2000Codec::Decode(DataElement const &in, DataElement &out)
      */
     //#ifdef SUPPORT_MULTIFRAMESJ2K_ONLY
     const SequenceOfFragments *sf = in.GetSequenceOfFragments();
-    assert( sf );
+    if( !sf ) return false;
     std::stringstream os;
-    assert( sf->GetNumberOfFragments() == Dimensions[2] );
+    if( sf->GetNumberOfFragments() != Dimensions[2] )
+      {
+      gdcmErrorMacro( "Not handled" );
+      return false;
+      }
     for(unsigned int i = 0; i < sf->GetNumberOfFragments(); ++i)
       {
       std::stringstream is;
@@ -384,9 +388,21 @@ bool JPEG2000Codec::Decode(std::istream &is, std::ostream &os)
       {
       gdcmWarningMacro( "BPP = " << comp->bpp << " vs BitsAllocated = " << PF.GetBitsAllocated() );
       }
-    assert( comp->sgnd == PF.GetPixelRepresentation() );
-    assert( comp->prec == PF.GetBitsStored());
+    if( comp->sgnd != PF.GetPixelRepresentation() )
+      {
+      PF.SetPixelRepresentation( comp->sgnd );
+      }
+#ifndef GDCM_SUPPORT_BROKEN_IMPLEMENTATION
+    assert( comp->prec == PF.GetBitsStored()); // D_CLUNIE_RG3_JPLY.dcm
     assert( comp->prec - 1 == PF.GetHighBit());
+#endif
+    //assert( comp->prec >= PF.GetBitsStored());
+    if( comp->prec != PF.GetBitsStored() )
+      {
+      PF.SetBitsStored( comp->prec );
+      PF.SetHighBit( comp->prec - 1 ); // ??
+      }
+    assert( PF.IsValid() );
     assert( comp->prec <= 32 );
 
     if (comp->prec <= 8)
@@ -492,6 +508,7 @@ opj_image_t* rawtoimage(char *inputbuffer, opj_cparameters_t *parameters,
     }
   if( bitsallocated % 8 != 0 )
     {
+    gdcmDebugMacro( "BitsAllocated is not % 8" );
     return 0;
     }
   assert( bitsallocated % 8 == 0 );
@@ -579,8 +596,6 @@ bool JPEG2000Codec::Code(DataElement const &in, DataElement &out)
   //
   // Create a Sequence Of Fragments:
   SmartPointer<SequenceOfFragments> sq = new SequenceOfFragments;
-  const Tag itemStart(0xfffe, 0xe000);
-  //sq->GetTable().SetTag( itemStart );
 
   const unsigned int *dims = this->GetDimensions();
 
@@ -603,7 +618,12 @@ bool JPEG2000Codec::Code(DataElement const &in, DataElement &out)
     const PixelFormat &pf = this->GetPixelFormat();
     int sample_pixel = pf.GetSamplesPerPixel();
     int bitsallocated = pf.GetBitsAllocated();
+#ifndef GDCM_SUPPORT_BROKEN_IMPLEMENTATION
     int bitsstored = pf.GetBitsStored();
+#else
+    // Usual D_CLUNIE_RG3_JPLY.dcm kludge:
+    int bitsstored = pf.GetBitsAllocated();
+#endif
     int sign = pf.GetPixelRepresentation();
     int quality = 100;
 
@@ -739,7 +759,6 @@ bool JPEG2000Codec::Code(DataElement const &in, DataElement &out)
     std::string str = os.str();
     assert( str.size() );
     Fragment frag;
-    //frag.SetTag( itemStart );
     frag.SetByteValue( &str[0], str.size() );
     sq->AddFragment( frag );
     }
@@ -859,6 +878,7 @@ bool JPEG2000Codec::GetHeaderInfo(const char * dummy_buffer, size_t buf_size, Tr
     gdcmErrorMacro( "Impossible happen" );
     return false;
     }
+  LossyFlag = !reversible;
 #if 0
 #ifndef GDCM_USE_SYSTEM_OPENJPEG
   if( j2k )
@@ -927,23 +947,27 @@ bool JPEG2000Codec::GetHeaderInfo(const char * dummy_buffer, size_t buf_size, Tr
     assert( image->color_space == 0 );
     //PI = PhotometricInterpretation::RGB;
     /*
-    8.2.4 JPEG 2000 IMAGE COMPRESSION
-    The JPEG 2000 bit stream specifies whether or not a reversible or irreversible multi-component (color)
-    transformation, if any, has been applied. If no multi-component transformation has been applied, then the
-    components shall correspond to those specified by the DICOM Attribute Photometric Interpretation
-    (0028,0004). If the JPEG 2000 Part 1 reversible multi-component transformation has been applied then
-    the DICOM Attribute Photometric Interpretation (0028,0004) shall be YBR_RCT. If the JPEG 2000 Part 1
-    irreversible multi-component transformation has been applied then the DICOM Attribute Photometric
-    Interpretation (0028,0004) shall be YBR_ICT.
-    Notes: 1. For example, single component may be present, and the Photometric Interpretation (0028,0004) may
-    be MONOCHROME2.
-    2. Though it would be unusual, would not take advantage of correlation between the red, green and blue
-    components, and would not achieve effective compression, a Photometric Interpretation of RGB could
-    be specified as long as no multi-component transformation was specified by the JPEG 2000 bit stream.
-    3. Despite the application of a multi-component color transformation and its reflection in the Photometric
-    Interpretation attribute, the color space remains undefined. There is currently no means of conveying
-    standard color spaces either by fixed values (such as sRGB) or by ICC profiles. Note in particular that
-    the JP2 file header is not sent in the JPEG 2000 bitstream that is encapsulated in DICOM.
+    8.2.4 JPEG 2000 IMAGE COMPRESSION 
+    The JPEG 2000 bit stream specifies whether or not a reversible or irreversible
+    multi-component (color) transformation, if any, has been applied. If no
+    multi-component transformation has been applied, then the components shall
+    correspond to those specified by the DICOM Attribute Photometric Interpretation
+    (0028,0004). If the JPEG 2000 Part 1 reversible multi-component transformation
+    has been applied then the DICOM Attribute Photometric Interpretation
+    (0028,0004) shall be YBR_RCT. If the JPEG 2000 Part 1 irreversible
+    multi-component transformation has been applied then the DICOM Attribute
+    Photometric Interpretation (0028,0004) shall be YBR_ICT.  Notes: 1. For
+    example, single component may be present, and the Photometric Interpretation
+    (0028,0004) may be MONOCHROME2.  2. Though it would be unusual, would not take
+    advantage of correlation between the red, green and blue components, and would
+    not achieve effective compression, a Photometric Interpretation of RGB could be
+    specified as long as no multi-component transformation was specified by the
+    JPEG 2000 bit stream.  3. Despite the application of a multi-component color
+    transformation and its reflection in the Photometric Interpretation attribute,
+    the color space remains undefined.  There is currently no means of conveying
+    standard color spaces either by fixed values (such as sRGB) or by ICC
+    profiles. Note in particular that the JP2 file header is not sent in the JPEG
+    2000 bitstream that is encapsulated in DICOM.
      */
     PI = PhotometricInterpretation::YBR_RCT;
     this->PF.SetSamplesPerPixel( 3 );
