@@ -3,7 +3,7 @@
   Program: GDCM (Grassroots DICOM). A DICOM library
   Module:  $URL$
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -13,11 +13,12 @@
 
 =========================================================================*/
 #include "gdcmPDFCodec.h"
+#include "gdcmTransferSyntax.h"
 
 int TestPDFCodec(int , char *[])
 {
   gdcm::PDFCodec c;
-  (void)c;
+  c.CanCode(gdcm::TransferSyntax());
 
   return 0;
 }
