@@ -1,9 +1,9 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-14/Utilities/VTK/Applications/gdcm2vtk.cxx $
+  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-15/Utilities/VTK/Applications/gdcm2vtk.cxx $
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -24,6 +24,8 @@
 #include "vtkImageData.h"
 #include "vtkMetaImageReader.h"
 #include "vtkMetaImageWriter.h"
+#include "vtkTIFFWriter.h"
+#include "vtkPNGWriter.h"
 #if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
 #include "vtkMINCImageReader.h"
 #include "vtkMINCImageAttributes.h"
@@ -56,7 +58,7 @@
 void PrintVersion()
 {
   std::cout << "gdcm2vtk: gdcm " << gdcm::Version::GetVersion() << " ";
-  const char date[] = "$Date: 2009-10-24 21:19:36 +0200 (Sat, 24 Oct 2009) $";
+  const char date[] = "$Date: 2010-06-23 18:15:58 +0200 (Wed, 23 Jun 2010) $";
   std::cout << date << std::endl;
   //std::cout << "             VTK " << vtkVersion::GetVTKVersion() << std::endl;
   std::cout << "          " << vtkVersion::GetVTKSourceVersion() << std::endl;
@@ -354,6 +356,23 @@ int main(int argc, char *argv[])
       writer->Write();
       return 0;
       }
+    else if(  gdcm::System::StrCaseCmp(outputextension,".png") == 0 )
+      {
+      vtkPNGWriter * writer = vtkPNGWriter::New();
+      writer->SetFileName( outfilename );
+      writer->SetInput( imgdata );
+      writer->Write();
+      return 0;
+      }
+    else if(  gdcm::System::StrCaseCmp(outputextension,".tif") == 0  
+      ||  gdcm::System::StrCaseCmp(outputextension,".tiff") == 0  ) // 
+      {
+      vtkTIFFWriter * writer = vtkTIFFWriter::New();
+      writer->SetFileName( outfilename );
+      writer->SetInput( imgdata );
+      writer->Write();
+      return 0;
+      }
     else if(  gdcm::System::StrCaseCmp(outputextension,".vti") == 0  ) // vtkXMLImageDataWriter::GetDefaultFileExtension()
       {
       vtkXMLImageDataWriter * writer = vtkXMLImageDataWriter::New();
