@@ -1,9 +1,9 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-14/Utilities/VTK/Applications/gdcmviewer.cxx $
+  Module:  $URL: https://gdcm.svn.sourceforge.net/svnroot/gdcm/tags/gdcm-2-0-15/Utilities/VTK/Applications/gdcmviewer.cxx $
 
-  Copyright (c) 2006-2009 Mathieu Malaterre
+  Copyright (c) 2006-2010 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -51,6 +51,7 @@
 #include "vtkOrientedGlyphContourRepresentation.h"
 #include "vtkPointHandleRepresentation2D.h"
 #include "vtkDistanceRepresentation2D.h"
+#include "vtkLegendScaleActor.h"
 #include "vtkProperty2D.h"
 #else
 class vtkLogoWidget;
@@ -626,6 +627,9 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
   iren->AddObserver(vtkCommand::EndPickEvent,obs);
   obs->Delete();
 
+  //vtkLegendScaleActor *legend = vtkLegendScaleActor::New();
+  //viewer->GetRenderer()->AddActor( legend );
+
   iren->Initialize();
   iren->Start();
 
@@ -678,7 +682,7 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
 void PrintVersion()
 {
   std::cout << "gdcmviewer: gdcm " << gdcm::Version::GetVersion() << " ";
-  const char date[] = "$Date: 2009-05-19 15:59:08 +0200 (Tue, 19 May 2009) $";
+  const char date[] = "$Date: 2010-06-23 18:15:58 +0200 (Wed, 23 Jun 2010) $";
   std::cout << date << std::endl;
   //std::cout << "             VTK " << vtkVersion::GetVTKVersion() << std::endl;
   std::cout << "            " << vtkVersion::GetVTKSourceVersion() << std::endl;
