@@ -3,7 +3,7 @@
 #  Program: GDCM (Grassroots DICOM). A DICOM library
 #  Module:  $URL$
 #
-#  Copyright (c) 2006-2009 Mathieu Malaterre
+#  Copyright (c) 2006-2010 Mathieu Malaterre
 #  All rights reserved.
 #  See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 #
@@ -23,6 +23,7 @@ blacklist = (
 "_j2k" # :)
 "_jp2" # :)
 "treamimpl" # :)
+"TestDriver"
 # DataStructureAndEncodingDefinition
 "ByteBuffer" # WTF ?
 "ExplicitDataElement"
@@ -53,7 +54,7 @@ blacklist = (
 "Table"
 "XMLPrivateDictReader"
 # Common
-"Macro"
+"LegacyMacro"
 "Swapper"
 "SmartPointer"
 "Win32"
@@ -81,6 +82,7 @@ blacklist = (
 "EncapsulatedDocument"
 "JPEGCodec"
 "PVRGCodec"
+"KAKADUCodec"
 "JPEGLSCodec"
 "PNMCodec"
 "PDFCodec"
